
package org.llvm.adt.aliases;

import org.clank.support.Converted;
import org.llvm.adt.DenseMapInfo;
import org.clank.support.*;

/**
 * class used to manage keys in Set/Map collections.
 * ${JAVA_CLASS_DESCRIPTION}
 *
 * @author Petr Kudryavtsev <petrk@netbeans.org>
 * @author Vladimir Voskresensky
 */
//<editor-fold defaultstate="collapsed" desc="<anonymous namespace>::PTHStringLookupTrait">
@Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/DenseMapInfo.h", line = 23,
 cmd = "jclank.sh -java-options=${SPUTNIK}/modules/org.llvm.adtsupport/llvmToClangType ${LLVM_SRC}/llvm/lib/Support/SmallPtrSet.cpp -filter=llvm::DenseMapInfo")
//</editor-fold>
public class ${FILE_NAME}${CLASS_SUFFIX} ${IMPLEMENTS_DENSE_MAP_INFO}{
  private static final ${FILE_NAME} $INFO = new ${FILE_NAME}();
  
  protected final ${RAW_TYPE} emptyKey;
  protected final ${RAW_TYPE} tombstoneKey;
  
  public static ${CLASS_SUFFIX} ${FILE_NAME}${TYPE_SUFFIX} $Info() {
    return $INFO;
  }

  public ${FILE_NAME}() {
    this(${DEFAULT_EMPTY_KEY}, ${DEFAULT_TOMBSTONE_KEY});
  }
  public ${FILE_NAME}(${RAW_TYPE} emptyKey, ${RAW_TYPE} tombstoneKey) {
    this.emptyKey = emptyKey;
    this.tombstoneKey = tombstoneKey;
    assert this.tombstoneKey != this.emptyKey : "emptyKey [" + this.emptyKey + "] must be different from tombstoneKey [" + this.tombstoneKey + "]";
    ${EXTRA_ASSERT_FOR_EMPTY_TOMBSTONE_KEYS}
  }

  public final/*inline*/ ${RAW_TYPE} getEmptyKey() {
    return emptyKey;
  }

  public final/*inline*/ ${RAW_TYPE} getTombstoneKey() {
    return tombstoneKey;
  }

  /**
   * @return return hashcode for the key (it is neither empty nor tombstone keys)
   */  
  public /*uint*/int getHashValue(/*const*/${TYPE} Key) {
    assert Key != emptyKey : "Key: must not be called for emptyKey";
    assert Key != tombstoneKey : "Key: must not be called for tombstoneKey";
    ${EXTRA_ASSERT_FOR_KEY}
    ${DEFAULT_HASH_VALUE_IMPL}
  }

  /**
   * IMPORTANT: hashcode value for AltKey must match hash-value for the original key.
   * @return return hashcode for the alternative key types (it is neither empty nor tombstone keys)
   */ 
  public /*uint*/int getHashValueAlt(/*const*/Object/*&*/ AltKey) {
${COMMENT_ALT}    assert AltKey != emptyKey : "AltKey: must not be called for emptyKey";
${COMMENT_ALT}    assert AltKey != tombstoneKey : "AltKey: must not be called for tombstoneKey";
    ${EXTRA_ASSERT_FOR_ALT_KEY}
    ${DEFAULT_HASH_VALUE_ALT_IMPL}    
  }

  /**
   * @return true when LHS key is equal to the RHS key (LHS/RHS are neither empty nor tombstone keys)
   */  
  public boolean isEqual(/*const*/${TYPE} LHS, /*const*/${TYPE} RHS) {
    assert LHS != emptyKey : "LHS: must not be called for emptyKey";
    assert LHS != tombstoneKey : "LHS: must not be called for tombstoneKey";    
    assert RHS != emptyKey : "RHS: must not be called for emptyKey";
    assert RHS != tombstoneKey : "RHS: must not be called for tombstoneKey";
    ${EXTRA_ASSERT_FOR_EQUAL_KEYS}
    ${DEFAULT_EQUAL_KEY_IMPL}
  }

  /**
   * @return true when alternative LHS key is equal to the stored RHS key (LHS/RHS is neither empty nor tombstone keys)
   */  
  public boolean isEqualAlt(/*const*/ Object /*&*/ AltKey, /*const*/ ${TYPE} /*&*/ Key) {
${COMMENT_ALT}    assert AltKey != emptyKey : "AltKey: must not be called for emptyKey";
${COMMENT_ALT}    assert AltKey != tombstoneKey : "AltKey: must not be called for tombstoneKey";
    assert Key != emptyKey : "Key: must not be called for emptyKey";
    assert Key != tombstoneKey : "Key: must not be called for tombstoneKey";
    ${EXTRA_ASSERT_FOR_KEY}
    ${EXTRA_ASSERT_FOR_ALT_KEY}
    ${DEFAULT_EQUAL_ALT_KEY_IMPL}
  }
  
  public boolean isKeyPointerLike() {
    ${DEFAULT_POINTER_LIKE_IMPL}
  }  

  protected boolean checkKey(Object Key) { return checkKeyImpl(Key, false); }
  protected boolean checkAltKey(Object AltKey) { return checkKeyImpl(AltKey, true); }

  protected boolean checkKeyImpl(Object Key, boolean altKey) {
    if (!NativeTrace.isDebugMode()) {
      NativeTrace.printStackTraceOnce(new Exception("must be called inside assert"));
      return true;
    }
    ${CHECK_KEY_CLASS_IMPL}
    return true;
  }
}
